const

FullScaleInt = $FFFFF;
FullScaleRealU = 1048575.0;
FullScaleReal = FullScaleRealU/2;
EEPROMSize = 64;

{ This constant is used to simplify calibration... calibration is done }
{ by multiplication, and then division (shift) by this number          }

TempScale = 10000;
FullScaleVolts5V = 5.0;
FullScaleVolts2r5V = 2.5;
FullScaleVolts1V = 1.0;
FullScaleVolts100mV = 0.100;
FullScaleVolts50mV = 0.050;
FullScaleVolts25mV = 0.025;
{ These set the calibrated data scale to microvolts }
Scale5V : real = 1E6 * FullScaleVolts5V / FullScaleReal;
Scale2r5V : real = 1E6 * FullScaleVolts2r5V / FullScaleReal;
Scale1V : real = 1E6 * FullScaleVolts1V / FullScaleReal;
Scale100MV : real = 1E6 * FullScaleVolts100mV / FullScaleReal;
Scale50MV : real = 1E6 * FullScaleVolts50mV / FullScaleReal;
Scale25MV : real = 1E6 * FullScaleVolts25mV / FullScaleReal;
ZeroChannel : byte = 128;
TempChannel : byte = 129;

TheCrecLen = EEPROMSize;
TheCrecType = $BABE;
TheCardRev  = $0001;
TheCalibrateSeal = $FADEDFAD;

A23BaseAdd0 = $220;
A23BaseAdd1 = $222;
A23BaseAdd2 = $224;
A23BaseAdd3 = $226;

{ Data Register Bits @ 4A23BaseAddress }

SClk     = $01;
DIn      = $02;
DOut     = $02;
ACS      = $04;
ECS      = $08;

{ Setup Register Bits @ 4A23BaseAddress + 1 -- bits 0..2 are IRQ select bits }

{ Setup register Interrupt select bytes, index by IRQ# FF byte means invalid 4A23 interrupt}
IRQSels  : Array [2..15] of byte =($00,$01,$02,$03,$FF,$04,$FF,$00,$05,$06,$07,$FF,$FF,$FF);

PSEn     = $08;
IEn      = $10;
IMask    = $20;


FirstChannel = 0;
LastChannel = 15;
LastUserChannel = LastChannel;
MaxSeqLength = 512;
DataFIFOLength = 64;
RoseBud = $defaced;

type

ADSource = (Normal,Ground,Temperature,Reverse);

{ This record represents the data stored in the serial EEPROM on the  }
{ 4A23 card. There should be exactly 128 bytes in this record         }
{ (the EEPROM is completely filled). Do not change this record if you }
{ expect to get along with future MESA 4A23 software!                 }

CalibRecordType =
record
  { Bipolar gain and offset calibration reg values}
  Gain5V        : longint;
  Offset5V      : longint;
  Gain2r5V      : longint;
  Offset2r5V    : longint;
  Gain1V        : longint;
  Offset1V      : longint;
  Gain100MV     : longint;
  Offset100MV   : longint;
  Gain50MV      : longint;
  Offset50MV    : longint;
  Gain25MV      : longint;
  Offset25MV    : longint;
  { Unipolar gain and offset calibration reg values }
  { note that the present software only uses Bipolar mode }
  Gain5VU       : longint;
  Offset5VU     : longint;
  Gain2r5VU     : longint;
  Offset2r5VU   : longint;
  Gain1VU       : longint;
  Offset1VU     : longint;
  Gain100MVU    : longint;
  Offset100MVU  : longint;
  Gain50MVU     : longint;
  Offset50MVU   : longint;
  Gain25MVU     : longint;
  Offset25MVU   : longint;
  TempGain      : longint;
  TempOff       : longint;
  CalibrateTemp : longint;
  CalDateY      : word;
  CalDateM      : byte;
  CalDateD      : byte;
  CalTimes      : byte;
  CalBy         : string[6];    { mainly padding... }
  CardRev       : word;
  CRecType    : word;
  CRecLen     : word;
  CheckSum      : word;
end;

CalRecPtrType = ^CalibRecordType;

type IndexedCalibRecordType =
record
  words: array[0..63] of word;
end;

