
Program ConfigDriver4A23;
uses dos;
{$I 4A23LOW }
{$I 8259 }
{$I Parse }
{$I HREAD }
{$I 4A23DLOW}
{$I 4A23DFUN}

type
DPtrRec = record
           Ofs : word;
           Smt : word;
         end;
var

IntRec : IntRecType;
IntRecPtr : ^IntRecType;
DriverInt : byte;
SeqFileName : string;
SeqFile : text;
AtoDRecPtr : AtoDRecPtrType;
SP,TP,TheChan : byte;
TheRange : longint;
TheDest : DataDestType;
Tokens : array[1..3] of string[20];
ValidRead,UserToken : boolean;
LoopToken : boolean;

procedure LoadSequence(asp,chan: byte;range:longint;dd : DataDestType);
begin
{  writeln('About to LoadSeq with sp = ',asp);}
  with AtoDRecPtr^ do
  begin
    if asp <= MaxSeqLength then
    begin
      SequenceTable[asp].Channel := chan;
      SequenceTable[asp].Range := range;
      SequenceTable[asp].DataDest := dd;
    end
    else writeln('Attempt to write beyond end of Sequence Table');
  end;
{  writeln('Load sequence - exit');}
end;

function OpenForRead(var fp: text; name: string): boolean;
begin
  Assign(fp,Name);
  {$I-}
  Reset(fp);
  {$I+}
  OpenForRead := IOResult = 0;
end { Open };

function GetChannel(astring : string; var anumber : byte): boolean;
var
code : integer;
begin
  GetChannel := false;
  StripLeadingBlanks(astring);
  StripTrailingBlanks(astring);
  if (length(astring) >0) then
  begin
    val(astring,anumber,code);
    if (code = 0) and (anumber >= FirstChannel) and (anumber <= LastChannel) then GetChannel := true;
  end;
end; { GetChannel }

function GetRange(astring : string; var range : longint): boolean;
begin
  range := 9999;
  if astring =  '5V'    then range := ADRange5V;
  if astring =  '2.5V'  then range := ADRange2r5V;
  if astring =  '1V'    then range := ADRange1V;
  if astring =  '100MV' then range := ADRange100mV;
  if astring =  '50MV'  then range := ADRange50mV;
  if astring =  '25MV'  then range := ADRange25mV;
  if range = 9999 then GetRange := false else GetRange := true;
end;

function GetDest(astring : string; var dest : DataDestType): boolean;
begin
  GetDest := false;
  if length(astring) <> 0 then
  begin
    if astring = 'PUSHDATA' then
    begin
      dest := PushData;
      GetDest := true;
    end;
    if astring = 'PUSHRDATA' then
    begin
      dest := PushRData;
      GetDest := true;
    end;
    if astring = 'PUSHZERO' then
    begin
      dest := PushZero;
      GetDest := true;
    end;
    if astring = 'PUSHTEMP' then
    begin
      dest := PushTemp;
      GetDest := true;
    end;
    if astring = 'TOSS' then
    begin
      dest := Toss;
      GetDest := true;
    end;
    if astring[1] = ';' then
    begin
      GetDest := true;
      dest := NoOp;
    end;
  end;
end;

procedure BumOut(s: string);
begin
  writeln;
  writeln(s);
  halt(2);
end;

procedure CallDriverInt(intno: byte;p :pointer);
begin
{  writeln('About to call SW int');}
  asm
    mov al,intno             { bad bad nasty old self modifying code ! }
    lea bx,@intbyte          { have heard that a better way is to }
    mov cs:[bx],al           { build the interrupt instruction and a ret }
    mov cx,DPtrRec[p].Smt    { on a stack frame and then call it there }
    mov bx,DPtrRec[p].Ofs    { maybe if i get ambitious later on...}
    jmp @intins              { we hope this dumps the queue! }
    @intins:                 { ins queues on 386 and above can be 16 bytes! }
    db $cd
    @IntByte:
    db $00
  end;
{  writeln('Call SW int - exit');}
end;

function DriverInstalledQ : boolean;
begin
{  writeln('About to check for driver installed');}
  DriverInstalledQ := false;
  IntRec.Data := GetD4A23Version xor $FFFFFFFF;
  IntRec.FunctionNum := DRFuncGetVersion;
  IntRec.ErrorCode := ErrorCodeBad;
  CallDriverInt(DriverInt,IntRecPtr);
  if IntRec.ErrorCode = ErrorCodeOK then
  begin
    if IntRec.Data = GetD4A23Version then DriverInstalledQ := true;
  end;
{  writeln('driver installed check - exit');}
end;

function StartConversions : boolean;
begin
  IntRec.FunctionNum := DRFuncStartConversions;
  IntRec.ErrorCode := ErrorCodeBad;
  CallDriverInt(DriverInt,IntRecPtr);
  if IntRec.ErrorCode = ErrorCodeOK
  then StartConversions := true
  else StartConversions := false;
end;

function StopConversions : boolean;
begin
{  writeln('about to stop conversions');}
  IntRec.FunctionNum := DRFuncStopConversions;
  IntRec.ErrorCode := ErrorCodeBad;
  CallDriverInt(DriverInt,IntRecPtr);
  if IntRec.ErrorCode = ErrorCodeOK
  then StopConversions := true
  else StopConversions := false;
{  writeln('stop conversions - exit');}
end;

function GetAtoDRecPtr : pointer;
begin
{  writeln('about to get AtoDRecPtr');}
  IntRec.FunctionNum := DRFuncGetAtoDRecPtr;
  IntRec.ErrorCode := ErrorCodeBad;
  CallDriverInt(DriverInt,IntRecPtr);
  GetAToDRecPtr := IntRec.Pointer;
{  writeln('get AtoDRecPtr - exit');}
end;

function GetParms : boolean;
var
wtemp : word;
begin
  GetParms := false;
  if length(ParamStr(1)) <> 0 then
  begin
    SeqFileName := ParamStr(1);
    if length(ParamStr(2)) <> 0 then
    begin
      if not HexWordRead(ParamStr(2),wtemp) then BumOut('Undecipherable software interrupt number');
      if (wtemp < $60) or (wtemp > $6F) then BumOut('Out of range software interrupt number');
      DriverInt := byte(wtemp);
      GetParms := true;
    end;
  end;
end;

procedure PushParse;
begin
  if GetChannel(Tokens[2],TheChan) then
  begin
    if GetRange(Tokens[3],TheRange) then
    begin
      LoadSequence(SP,TheChan,TheRange,TheDest);
      SP := SP +1;
      UserToken := true;
    end
    else
    begin
      writeln('FATAL: Bad range value: ',Tokens[3]);
      ValidRead := false;
    end;
  end
  else
  begin
    writeln('FATAL: Bad channel number: ',Tokens[2]);
    ValidRead := false;
  end;
end; {PushParse}

procedure ZeroParse;
begin
  if GetRange(Tokens[2],TheRange) then
  begin
    LoadSequence(SP,ZeroChannel,TheRange,TheDest);
    SP := SP +1;
    UserToken := true;
  end
  else
  begin
    writeln('FATAL: Bad range value: ',Tokens[3]);
    ValidRead := false;
  end;
end; {GenReadParse}

procedure  TempParse;
begin
  LoadSequence(SP,TempChannel,0,TheDest);
  SP := SP +1;
  UserToken := true;
end;


procedure TossParse;
begin
  if GetChannel(Tokens[2],TheChan) then
  begin
    if GetRange(Tokens[3],TheRange) then
    begin
      LoadSequence(SP,TheChan,TheRange,TheDest);
      SP := SP +1;
    end
    else
    begin
      writeln('FATAL: Bad range value: ',Tokens[3]);
      ValidRead := false;
    end;
  end
  else
  begin
    writeln('FATAL: Bad channel number: ',Tokens[2]);
    ValidRead := false;
  end;
end;

procedure  ParseDest;
begin
  case TheDest of
    PushData,PushRdata  : PushParse;
    PushZero  : ZeroParse;
    PushTemp  : TempParse;
    Toss      : TossParse;
  end;
end;

function ParseParm : boolean;
var
wtemp : longint;
retcode : integer;
begin

  ParseParm := false;
  with AtoDRecPtr^ do
  begin
    if Tokens[1] = 'SETRATE' then
    begin
      ParseParm := true;
      ConversionRate := 9999;
      if Tokens[2] = '3.76' then ConversionRate := ADWordRate3r76;
      if Tokens[2] = '7.51' then ConversionRate := ADWordRate7r51;
      if Tokens[2] = '15' then ConversionRate := ADWordRate15;
      if Tokens[2] = '30.1' then ConversionRate := ADWordRate30;
      if Tokens[2] = '60' then ConversionRate := ADWordRate60;
      if Tokens[2] = '123.2' then ConversionRate := ADWordRate123;
      if Tokens[2] = '168.9' then ConversionRate := ADWordRate169;
      if Tokens[2] = '202.3' then ConversionRate := ADWordRate202;
      if ConversionRate = 9999 then
      begin
        writeln('FATAL: Bad SetRate Value: ',Tokens[2]);
        ValidRead := false;
      end;
    end;
    if Tokens[1] = 'LOOPSTART' then
    begin
      if LoopToken = false then
      begin
        ParseParm := true;
        SequenceStart := SP;
        LoopToken := true;
      end
      else
      begin
        writeln('FATAL: More than one LoopStart in table');
        ValidRead := false;
      end;
    end;
  end;
end;


function ReadSequenceFile : boolean;
var
seqstr : string;

begin
  ValidRead := true;
  UserToken := false;      { user data statement encountered }
  LoopToken := false;
  SP := 1;
  if OpenForRead(SeqFile,SeqFileName) then
  begin
    while not EOF(SeqFile) do
    begin
      Tokens[1] := '  ';
      Tokens[2] := '  ';
      Tokens[3] := '  ';
      readln(SeqFile,seqstr);
      if length(seqstr) > 0 then
      begin
        seqstr := UpString(seqstr);
        StripLeadingBlanks(seqstr);
        StripTrailingBlanks(seqstr);
        if length(seqstr) > 0 then
        begin
          tp := 1;
          while length(seqstr) >0 do
          begin
            Tokens[tp] := strip(seqstr);
            tp := tp +1;
            if (tp > 3) then tp := 3;
          end;
          if GetDest(Tokens[1],TheDest) then ParseDest
          else if not ParseParm then
          begin
            writeln('WARNING: Invalid function: ''',Tokens[1],'''');
          end;
        end; {if length >0 (2)}
      end; { if length >0 (1)}
    end; { while not EOF }
  end  { if OpenForRead(SeqFile)}
  else
  begin
    writeln('FATAL: No such sequence file: ',SeqFileName);
    ValidRead := false;
  end;
  if ValidRead then
  begin
    if not LoopToken then
    begin
      writeln('FATAL: No LoopStart entry in table!');
      ValidRead := false;
    end;
    if not UserToken then
    begin
      writeln('FATAL: No USER data entry in table!');
      ValidRead := false;
    end; {if not... }
  end;
  if SP-1 > MaxSeqLength then
  begin
    writeln('FATAL: Sequence is longer that allowed');
    ValidRead := false;
  end;
  if ValidRead then AtoDRecPtr^.SequenceEnd := SP-1;
  ReadSequenceFile := ValidRead;
  AtoDRecPtr^.AtoDRecSeal := RoseBud;
end; { Read Sequence File }

procedure Usage;
begin
  writeln;
  writeln('USAGE: 4A23DCFG  FILE  SOFT');
  writeln;
  writeln('PURPOSE: Configures 4A23 Analog card driver');
  writeln;
  writeln('Where FILE is the A-D read sequence file');
  writeln('and   SOFT is the (hex) software interrupt used by 4A23DRVR');
  writeln;
  halt(2);
end;

begin                  { main }
  IntRecPtr := @IntRec;
  if GetParms then
  begin
{    writeln('IntRecPtr = ',PtrRec(IntRecPtr).Smt,':',PtrRec(IntRecPtr).Ofs);
     if IntNullPtrQ(DriverInt) then BumOut('4A23 driver not found!-NP');}
    if not DriverInstalledQ then BumOut('4A23 driver not found!');
{    writeln('Driver is installed!');}
    AtoDRecPtr := GetAtoDRecPtr;
{    writeln('AtoDRecPtr = ',PtrRec(AtoDRecPtr).Smt,':',PtrRec(AtoDRecPtr).Ofs);
    writeln('About to install defaults'); }
{    writeln('Defaults installed');}
    if ReadSequenceFile then
    begin
      if StopConversions then
      begin
        writeln('4A23 Driver Configured...');
      end
      else
      begin
        writeln('Shouldn''t...');
      end;
    end;
  end
  else Usage;
end.
