type
DataDestType = (Toss,PushData,PushRdata,PushZero,PushTemp,NoOp);


SequenceTableType   =
record
  Channel       : byte;
  Range         : LongInt;
  DataDest      : DataDestType;
end;

DataEntryType =
record
  Data      : longint;
  Channel   : byte;
  Range     : longint;
  { timestamp? }
end;

DataFIFOType =
record
  PutPointer : word;
  GetPointer : word;
  OverflowQ  : boolean;
  FIFOData : array[1.. DataFIFOLength] of DataEntryType;
end;

AtoDRecordType =
record
  DAveTimes       : word;     { number of times to average user data reading }
  DResetStep      : word;     { delta in sequential data readings needed to reset averager }
  ScanCount       : longint;  { number of total scan cycles }
  CardTemperature : longint;  { calibrated temp }
  ConversionRate  : longint;  { conversion rate }
  CurrentData     : longint;  { most recent A-D reading }
  SWIntNo         : byte;     { the Software interrupt number the driver is using }
  HWIntNo         : byte;     { the Hardware interrupt number the driver is using }
  ConvRunning     : boolean;  { whether the converter is running or not }
  DataToPush      : DataEntryType;        { an image of the FIFO record }
  SequenceStart   : byte;
  SequenceEnd     : byte;
  SeqPointer      : byte;
  SequenceTable   : array [1..MaxSeqLength] of SequenceTableType;
  CalibrateData   : CalibRecordType;
  AtoDRecSeal     : longint;
end;

AtoDRecPtrType = ^AtoDRecordType;
DataFIFOPtrType = ^DataFIFOType;
DataEntryPtrType = ^DataEntryType;

function ADFIFOInc(dptr : word): word;
begin
  if (dptr + 1) > DataFIFOLength then dptr := 1
  else dptr := dptr +1;
  ADFIFOInc := dptr;
end;

procedure InitADFIFO(dfp : DataFIFOPtrType);
begin
  with dfp^ do
  begin
    PutPointer := 1;
    GetPointer := 1;
    OverFlowQ := false;
  end;
end;

function ADFIFOHasData(dfp : DataFIFOPtrType) : boolean;
begin
  ADFIFOHasData := false;
  with dfp^ do
  begin
    if GetPointer <> PutPointer then ADFIFOHasData := true;
  end;
end;

function ADFIFOIsFull(dfp : DataFIFOPtrType) : boolean;
begin
  ADFIFOIsFull := false;
  with dfp^ do
  begin
    { if hypothetical -put- would smash -get- data then fifo is full! }
    if ADFIFOInc(PutPointer) = GetPointer then ADFIFOIsFull := true;
  end;
end;

function ADFIFOGet(dfp : DataFIFOPtrType): DataEntryPtrType;
begin
  if ADFIFOHasData(dfp) then
  begin
    with dfp^ do
    begin
      ADFIFOGet := @FIFOData[GetPointer];
      GetPointer := ADFIFOInc(GetPointer);
    end;
  end;
end;

procedure ADFIFOPut(dfp : DataFIFOPtrType; dep : DataEntryPtrType);

begin
  with dfp^ do
  begin
    if (not ADFIFOIsFull(dfp)) then
    begin
      FIFOData[PutPointer] := dep^;        { and put the new data there }
      PutPointer := ADFIFOInc(PutPointer);
    end
    else
    begin
      OverFlowQ := true;                   { push with no room so ... }
      GetPointer := ADFIFOInc(GetPointer); { throw away the oldest data }
      FIFOData[PutPointer] := dep^;        { and put the new data there }
      PutPointer := ADFIFOInc(PutPointer);
    end;
  end;
end;
