Program DRVR4A23;
uses dos;
{$I 8259 }
{$I 4A23low}

{$M 4096,32768,32768}
{{$r-}
{{$s-}
{$I 4A23DLOW}
{$I 4A23DFUN}
var
DataFIFO : DataFIFOType;
AtoDRecord : AtoDRecordType;
CalibRecord : CalibRecordType;
IntRecPtr : ^IntRecType;

AtoDRecPtr : AtoDRecPtrType;
CalRecPtr : CalRecPtrType;
DataFIFOPtr : DataFIFOPtrType;
DataToPushPtr : DataEntryPtrType;

procedure ResetScan;
begin
  with AtoDRecord do
  begin
    ScanCount       := 0;
    ConvRunning     := False;      { not running }
    SeqPointer      := 1;
  end;
  InitADFIFO(DataFIFOPtr);
end;

function NextSequence(sq : byte): byte;
begin
  with AtoDRecord do
  begin
    if sq = SequenceEnd then
    begin
      NextSequence := SequenceStart;
    end
    else NextSequence := sq + 1;
  end;
end;

procedure AdvanceSequence;
begin
  with AtoDRecord do
  begin
    if SeqPointer = SequenceEnd then ScanCount := ScanCount +1;
    SeqPointer := NextSequence(SeqPointer);
  end;
end;

{**********************************************************************}
{              Our software interrupt functions                        }
{**********************************************************************}

procedure GetVersion;
begin
  IntRecPtr^.Data := GetD4A23Version;
  IntRecPtr^.ErrorCode := 0;
end;

procedure GetDriverStatus;
begin
  IntRecPtr^.Status := StTrue;  { dummy function for now }
  IntRecPtr^.ErrorCode := 0;
end;

procedure GetConvStatus;
begin
  if AtoDRecord.ConvRunning then IntRecPtr^.Status := StTrue
                            else IntRecPtr^.Status := StFalse;
  IntRecPtr^.ErrorCode := 0;
end;

procedure GetFIFODataStatus;
begin
  if ADFIFOHasData(DataFIFOPtr)  then IntRecPtr^.Status := StTrue
                                 else IntRecPtr^.Status := StFalse;
  IntRecPtr^.ErrorCode := 0;
end;

procedure GetFIFOOvflStatus;
begin
  if DataFIFOPtr^.OverflowQ = true then
  begin
    IntRecPtr^.Status := StTrue;
    DataFIFOPtr^.OverflowQ := false;
  end
  else IntRecPtr^.Status := StFalse;
  IntRecPtr^.ErrorCode := 0;
end;

procedure StartConversions;
var
dummy : longint;
begin
  with AtoDRecord do
  begin
    if ConvRunning = false then
    begin
      ResetScan;
      SetConvRate(ADWordRate60);
      SetRangeAndCal(ADRange5V,@CalibRecord);
      SetChannel(0);
      Dummy := PolledReadAD;
      DisableInterrupts;
      MaskADInt;
      SetConvRate(AtoDRecord.ConversionRate);
      MaskInterrupt(AtoDRecord.HWIntNo);
      { setup for first conversion (SeqPointer = 1 here) }
      case SequenceTable[SeqPointer].DataDest of
        PushData:
        begin
          SetInputSource(Normal);
          SetChannel(SequenceTable[SeqPointer].Channel);
          SetRangeAndCal(SequenceTable[SeqPointer].Range,@CalibRecord);
        end;
        PushRData:
        begin
          SetInputSource(Reverse);
          SetChannel(SequenceTable[SeqPointer].Channel);
          SetRangeAndCal(SequenceTable[SeqPointer].Range,@CalibRecord);
        end;
        PushZero:
        begin
          SetInputSource(Ground);
          SetRangeAndCal(SequenceTable[SeqPointer].Range,@CalibRecord);
        end;
        PushTemp:
        begin
          SetInputSource(Temperature);
          SetRangeAndCal(ADRange1V,@CalibRecord);
        end;
      end; {case }
      {start the ball rolling by sending a start convert command }
      ADSendCommand(SingleConvertCom);
      UnmaskInterrupt(AtoDRecord.HWIntNo);
      IssueEOI;
      UnMaskADInt;
      EnableInterrupts;
      ConvRunning := true;
      IntRecPtr^.ErrorCode := 0;
    end;
  end;
end;

procedure StopConversions;
begin
  ResetScan;
  MaskADInt;
  IntRecPtr^.ErrorCode := 0;
end;


procedure GetFIFOData;
begin
  with IntRecFIFOType(IntRecPtr^) do
  begin
    { data is grabbed with interrupts disabled                            }
    DataRec := ADFIFOGet(DataFIFOPtr)^;
    with DataRec do
    begin
      EnableInterrupts;
      { We enable interrupts here because CalibrateData may take a while... }
      Data := Data;
    end;
    ErrorCode := 0;
  end;
end;

procedure GetScanCount;
begin
  IntRecPtr^.Data := AtoDRecord.ScanCount;
  IntRecPtr^.ErrorCode := 0;
end;

procedure GetAtoDRecPtr;
begin
  IntRecPtr^.Pointer := AtoDRecPtr;
  IntRecPtr^.ErrorCode := 0;
end;

procedure GetCalibRecPtr;
begin
  IntRecPtr^.Pointer := CalRecPtr;
  IntRecPtr^.ErrorCode := 0;
end;

procedure DriverSWInt(Flags,CS,IP,AX,BX,CX,DX,SI,DI,DS,ES,BP: word);
interrupt;
var fnum : byte;
begin
  { Notice that at this point interrupts are disabled.      }
  { We leave them disabled to prevent our ISR from changing }
  { our data as we read it. We do enable interrupts when    }
  { we call the CalibrateData procedure due to its length...}
  PtrRec(IntRecPtr).Seg := CX;
  PtrRec(IntRecPtr).Ofs := BX;
  IntRecPtr^.ErrorCode := ErrorCodeBad;
  fnum := mem[CX:BX];
  case fnum of
    DrFuncGetVersion       : GetVersion;
    DrFuncGetConvStatus    : GetConvStatus;
    DrFuncGetDriverStatus  : GetDriverStatus;
    DrFuncGetFIFODataStatus: GetFIFODataStatus;
    DrFuncGetFIFOOvflStatus: GetFIFOOvflStatus;
    DrFuncStartConversions : StartConversions;
    DrFuncStopConversions  : StopConversions;
    DrFuncGetFIFOData      : GetFIFOData;
    DrFuncGetScanCount     : GetScanCount;
    DrFuncGetAtoDRecPtr    : GetAtoDRecPtr;
    DrFuncGetCalibRecPtr   : GetCalibRecPtr;
  end;
end;


{**********************************************************************}
{              Our hardware interrupt service routine                  }
{**********************************************************************}

procedure BackScanISR;
interrupt;
begin
    { We got here via a hardware (EOC) interrupt, the A-D is still running, }
    { so we need to get the data and set up the A-D for the next conversion quickly! }
    { first set range, and read data from previous channel before new measurement begins }
    { we have about 3 milliseconds of integrate zero time in which to accomplish this }
    { set range and start conversion for next reading }

  with AtoDRecord do
  begin
    if ConvRunning then
    begin
      { This masking is very important since the interrupt will  }
      { get repetitively set and cleared when reading the data   }
      { since it is basically just the A-D serial data out bit!  }
      { if we did not mask at the A-D here, we would immediately }
      { get another interrupt as soon as we re-enabled interrupts}
      MaskADInt;
      { next we read the data }
      CurrentData := ISRReadAD;
      { then we set the channel for the next conversion}
      { set the channel and range number for the next conversion }
      case SequenceTable[NextSequence(SeqPointer)].DataDest of
        PushData:
        begin
          SetInputSource(Normal);
          SetChannel(SequenceTable[NextSequence(SeqPointer)].Channel);
          SetRangeAndCal(SequenceTable[NextSequence(SeqPointer)].Range,@CalibRecord);
        end;
        PushRData:
        begin
          SetInputSource(Reverse);
          SetChannel(SequenceTable[NextSequence(SeqPointer)].Channel);
          SetRangeAndCal(SequenceTable[NextSequence(SeqPointer)].Range,@CalibRecord);
        end;
        PushZero:
        begin
          SetInputSource(Ground);
          SetRangeAndCal(SequenceTable[NextSequence(SeqPointer)].Range,@CalibRecord);
        end;
        PushTemp:
        begin
          SetInputSource(Temperature);
          SetRangeAndCal(ADRange1V,@CalibRecord);
        end;
      end; {case }
      { start next Conversion }
      ADSendCommand(SingleConvertCom);
      { unmask AD int at 4A23 }
      UnMaskADInt;
      { now that we've got the next conversion started, decide what to }
      { do with the data }
      with SequenceTable[SeqPointer] do
      if DataDest <> Toss then
      begin
        begin
          DataToPush.Data := CurrentData;
          DataToPush.Channel := Channel;
          DataToPush.Range := Range;
          ADFIFOPut(DataFIFOPtr,DataToPushPtr);
        end; { PushData }
      end;
      AdvanceSequence;
    end; { if ConvRunning }
    if HWIntNo < 8 then IssueEOI else IssueEOI2;
  end; { with AtoDRecord }
end;

procedure BumOut(s: string);
begin
  writeln;
  writeln(s);
  halt(2);
end;

procedure InstallInts;
begin
{  writeln('About to install ints');}
  with AtoDRecord do
  begin
    DisableInterrupts;
    MaskADInt;
    SetHardIntVec(HWIntNo,@BackScanISR);
    SetIntVec(SWIntNo,@DriverSWInt);
    MaskInterrupt(HWIntNo);
    if SetIntNum(HWIntNo) then;
    EnableInterrupts;
  end;
{  Writeln('Done installing ints');}
end;

procedure FullInit;
begin
  if not Init4A23 then BumOut('Can''t Initialize 4A23');
  InstallInts;
  with AtoDRecord do
  begin
    ConvRunning := false;  { to stub isr }
  end;
  AtoDRecPtr    := @AtoDRecord;
  CalRecPtr     := @CalibRecord;
  DataFIFOPtr   := @DataFIFO;
  DataToPushPtr := @AtoDRecord.DataToPush;
  ResetScan;
end;

function HexRead(hexstring : string; var hexnumber : word): boolean;
var
hindex : byte;
charVal : integer;
placeVal : integer;
const
hexChars : string = '0123456789ABCDEF';
begin
  hexNumber :=0;
  HexRead := true;
  for hindex := 1 to length(hexstring) do
  begin
    hexNumber := HexNumber * $10;
    charval := pos(upcase(hexstring[hindex]),hexChars) -1;
    if charval < 0 then
    begin
      HexRead := false;
    end
    else
    begin
      hexnumber := hexnumber + charVal;
    end;
  end;
end;

procedure HexPrint(theNum : longint; nibbles : byte);
var
shiftCount : integer;
const
hexChars : array[0..15] of char = ('0','1','2','3','4','5','6','7',
                                   '8','9','A','B','C','D','E','F');
begin
shiftCount := (nibbles * 4) -4;
  while shiftCount >= 0 do
  begin
    write(hexChars[((thenum shr shiftCount) and $000F)]);
    shiftCount := shiftCount - 4;
  end;
end;

procedure Usage;
begin
  writeln;
  writeln('USAGE: 4A23DRVR  PORT  HARD  SOFT');
  writeln;
  writeln('PURPOSE: Interface to 4A23 analog input card');
  writeln;
  writeln('Where PORT is the (hex) port address of the 4A23');
  writeln('      HARD is the (decimal) hardware interrupt to use [2..15]');
  writeln('and   SOFT is the (hex) software interrupt to use [60..6F]');
  writeln;
  halt(2);
end;

function GetParams : boolean;
var
hint,sint : word;
retcode : integer;

begin
  if (length(ParamStr(3)) >0) and (length(ParamStr(4)) = 0) then
  begin
    if not HexRead(ParamStr(1),BasePort) then BumOut('Undecipherable port address');
    if (BasePort <> A23BaseAdd0) and (BasePort <> A23BaseAdd1) and (BasePort <> A23BaseAdd2) and(BasePort <> A23BaseAdd3) then
    Bumout('Out of range port address');
    val(ParamStr(2),hint,retcode);
    if retcode <>0 then BumOut('Undecipherable hardware interrupt number');
    if not CheckIntNum(hint) then BumOut('Invalid hardware interrupt');
    if not HexRead(ParamStr(3),sint) then Bumout('Undecipherable software interrupt number');
    if (sint < $60)  or (sint > $6F) then Bumout('Out of range sofware interrupt number');
    GetParams := true;
  end
  else GetParams := false;
  with AtoDRecord do
  begin
    if HWIntNo = 9 then HWIntNo := 2;
    HWIntNo := byte(hint);
    SWIntNo := byte(sint);
  end;
end;

begin
  if not GetParams then Usage;
  InitPortVars;
  if not A23InstalledQ then BumOut('Could not find 4A23 card');
{  if not IntNullPtrQ(AtoDRecord.SWIntNo) then Bumout('Software interrupt is occupied');}
  FullInit;
{  writeln('AtoDRecPtr = ',PtrRec(AtoDRecPtr).Smt,':',PtrRec(AtoDRecPtr).Ofs);}
  if not ReadCalData(CalRecPtr) then Bumout('4A23 not calibrated...');
  writeln('4A23 Analog card driver installed');
  Keep(0);
end.

