{ low level interface routines for the 4A23 }
{$I 4A23BITS}
{$I CS5526}


var

BasePort        : word;
DataPort        : word;
DataPortShadow  : byte;
SetupPort       : word;
SetupPortShadow : byte;
ConfigRegShadow : longint;
Bipolar : boolean;

procedure ADisableInterrupts;
begin
  inline($FA);                                 { CLI }
end;

procedure AEnableInterrupts;
begin
  inline($FB);                                 { STI }
end;

Procedure LocalDelay(d: word);
var
iloops,oloops : word;
dummy : byte;
begin
  for oloops := 0 to d do
  begin
    for iloops := 1 to 1000 do
    begin
      dummy := port[$61];
    end;
  end;
end;

procedure InitPortVars;
begin
  { this is done only so that port bit access runs faster }
  DataPort   := BasePort;
  SetupPort  := BasePort+1;
end;


{ These routines write to the control registers on the 4A23 }
{ they shadow the old registers since the registers are not }
{ all readable and shadowing saves a (slow) I/O instruction }

procedure WriteDataShadow(sdata : byte);
begin
  DataPortShadow := sdata;
  port[DataPort] := DataPortShadow;
end;

procedure WriteSetupShadow(sdata : byte);
begin
  SetupPortShadow := sdata;
  port[SetupPort] := SetupPortShadow;
end;

procedure SetCSHigh;
begin
  WriteDataShadow(DataPortShadow or ECS);
end;

procedure SetCSLow;
begin
  WriteDataShadow(DataPortShadow and (not ECS));
end;

procedure SetACSHigh;
begin
  WriteDataShadow(DataPortShadow or ACS);
end;

procedure SetACSLow;
begin
  WriteDataShadow(DataPortShadow and (not ACS));
end;

procedure SetDinHigh;
begin
  WriteDataShadow(DataPortShadow or DIn);
end;

procedure SetDinLow;
begin
  WriteDataShadow(DataPortShadow and (not DIn));
end;

procedure SetClockHigh;
begin
  WriteDataShadow(DataPortShadow or SCLK);
end;

procedure SetClockLow;
begin
  WriteDataShadow(DataPortShadow and (not SCLK));
end;

function DataHighQ : boolean;
begin
  { EEPROM data out }
  if (port[SetupPort] and DOut) = 0 then DataHighQ := false
  else DataHighQ := true;
end;

function ADataHighQ : boolean;
begin
  { CS5526 data out }
  if (port[DataPort] and DOut) = 0 then ADataHighQ := false
  else ADataHighQ := true;
end;

{$I 9346LOW}

procedure DisableIntDrive;
begin
  WriteSetupShadow(SetupPortShadow and (not IEn));
end;

procedure EnableIntDrive;
begin
  WriteSetupShadow(SetupPortShadow or IEn);
end;

function CheckIntNum(intno : byte) : boolean;
begin
  CheckIntNum := false;
  if (intno >= 2) and (intno <= 15) then
  begin
    if (IRQSels[intno] <> $FF) then CheckIntNum := true;
  end;
end;

function SetIntNum(intno : byte) : boolean;
begin
  SetIntNum := false;
  if CheckIntNum(intno) then
  begin
    { first disable interrupt driver so we don't cause a conflict }
    DisableIntDrive;
    { then write the Interrupt select code }
    WriteSetupShadow(SetupPortShadow or IRQSels[intno]);
    EnableIntDrive;
    SetIntNum := true;
  end;
end;

procedure MaskADInt;
begin
  WriteSetupShadow(SetupPortShadow and (not IMask));
end;

procedure UnMaskADInt;
begin
  WriteSetupShadow(SetupPortShadow or IMask);
end;


function A23InstalledQ: boolean;
begin
  A23InstalledQ := false;
  { first simple read check }
  if (port[DataPort] and $3D) = $15 then
  begin
    { then do write/read test of a couple bits... }
    WriteSetupShadow(0);
    if (port[SetupPort] and $38) = 0 then
    begin
      WriteSetupShadow(PSEn or IMask);
      if (port[SetupPort] and $38) = (PSEn or IMask) then A23InstalledQ := true;
    end;
  end;
end;

procedure ADSendCommand(com : byte);
var
cmask : byte;
index : byte;
begin
  cmask := $80;
  for index := 0 to 7 do { actually bit = 7..0 }
  begin
    if ((com and cmask) = 0) then SetDInLow else SetDInHigh;
    SetClockHigh;
    SetClockLow;
    cmask := cmask shr 1;
  end;
end;

procedure ADAccess(com : byte; var data : longint);
var
index : byte;
adata : longint;
amask : longint;
begin
  { Send Command First}
  ADSendCommand(com);
  { then read or write data }
  adata := 0;
  amask := $800000;
  if (com and ADReadBit) = 0 then
  begin { write case}
    for index := 0 to 23 do { actually bit = 23..0 }
    begin
      if (data and amask) = 0 then SetDInLow else SetDInHigh;
      SetClockHigh;
      SetClockLow;
      amask := amask shr 1;
    end;
    adata := data;
  end
  else
  begin { read case}
    for index := 0 to 23 do { actually bit = 23..0 }
    begin
      if ADataHighQ then adata := (adata or amask);
      SetClockHigh;
      SetClockLow;
      amask := amask shr 1;
    end;
  end;
  data := adata;
end;

function PFReadDataReg : longint;
{ for reading the data register directly without a command }
{ after the conversion is complete - this only works with PF bit set }
var
index : byte;
adata : longint;
amask : longint;
begin
  { first clear the PF bit with 8 clocks ,DIN low}
  SetDINLow;
  for index := 1 to 8 do
  begin
    SetClockHigh;
    SetClockLow;
  end;
  adata := 0;
  amask := $800000;
  begin { read case}
    for index := 0 to 23 do { actually bit = 23..0 }
    begin
      if ADataHighQ then adata := (adata or amask);
      SetClockHigh;
      SetClockLow;
      amask := amask shr 1;
    end;
  end;
  PFReadDataReg := adata;
end;

procedure WriteConfigShadow(sdata : longint);
begin
  ConfigRegShadow := sdata;
  ADAccess(WriteConfigCom,sdata);
end;

function ReadConfigReg : longint;
var
sdata : longint;
begin
  ADAccess(ReadConfigCom,sdata);
  ReadConfigReg := sdata;
end;

function ReadDataReg : longint;
var
sdata : longint;
begin
  ADAccess(ReadADDataCom,sdata);
  ReadDataReg := sdata;
end;

procedure SetChannel(chan: byte);
var
lchan : longint;
begin
  lchan := chan;
  WriteConfigShadow((ConfigRegShadow and $0FFFF) or (lchan shl 20));
end;

procedure SetInputSource(source : ADSource);
begin
  { Breaks channel number!}
  SetChannel(byte(source));
  SetACSHigh;
  SetACSLow;
end;

procedure SetRange(range: longint);
begin
  WriteConfigShadow((ConfigRegShadow and $FFF1FF) or range);
end;

procedure SetRangeAndCal(range: longint; calibrecptr: CalRecPtrType);
var
gain,offset : longint;
begin
  SetRange(range);
  with calibrecptr^ do
  begin
    { First get the calibration data for this range }
    case range of
      ADRange100MV: begin gain := Gain100MV ; offset := Offset100MV end;
      ADRange50MV:  begin gain := Gain50MV ; offset := Offset50MV end;
      ADRange25MV:  begin gain := Gain25MV ; offset := Offset25MV end;
      ADRange1V:    begin gain := Gain1V ; offset := Offset1V end;
      ADRange5V:    begin gain := Gain5V ; offset := Offset5V end;
      ADRange2r5V:  begin gain := Gain2r5V ; offset := Offset2r5V end;
    end; {case}
  end; {with}
  { Then write the calibration data to the CS5526s offset and gain registers }

  ADAccess(WriteOffsetCom,offset);
  ADAccess(WriteGainCom,gain);
end;

procedure SetConvRate(rate: longint);
begin
  WriteConfigShadow((ConfigRegShadow and $FF1FFF) or rate);
end;

procedure SetUnipolarMode;
begin
  Bipolar := false;
  WriteConfigShadow(ConfigRegShadow or ADUnipolar);
end;

procedure SetBipolarMode;
begin
  Bipolar := true;
  WriteConfigShadow(ConfigRegShadow and (not ADUnipolar));
end;

procedure SetPortFlagInActive;
begin
  WriteConfigShadow(ConfigRegShadow and (not ADPortFlag));
end;

procedure SetPortFlagActive;
begin
  WriteConfigShadow(ConfigRegShadow or ADPortFlag);
end;

function Init4A23 : boolean;
var
index : byte;
configdata : longint;
begin
  InitPortVars;
  Init4A23 := false;
  if A23InstalledQ then
  begin
    {Start from the reset condition }
    WriteSetupShadow(0);
    WriteDataShadow(0);
    {Turn off the power}
    LocalDelay(100);
    {Turn on the power}
    WriteSetupShadow(PSEn);
    {Wait for power to stabilize on isolated side}
    LocalDelay(100);
    {Warble the isolated bits to get synchronized }
    { all low }
    WriteDataShadow(0);
    { all high}
    WriteDataShadow(SCLK or DIn or ACS);
    {all low again}
    WriteDataShadow(0);
    {Do serial init on CS5526 (DS page 20) }
    { a long string of FFs followed by one FE:}
    SetDinHigh;
    for index := 1 to 128 do
    begin
      if index = 128 then SetDinLow;
      SetClockHigh;
      SetClockLow;
    end;
    {Now do a simple write-read test of the config register bits}
    {this selects channel 15...}
    WriteConfigShadow($F00000);
    configdata :=ReadConfigReg;
    {First read may fail because its the first changing data back from A-D}
    WriteConfigShadow($F00000);
    configdata := ReadConfigReg;
    { we mask off the read-only done flag...}
    if (configdata and $FFFFF7)= $F00000 then
    begin
      WriteConfigShadow($000000);
      configdata:= ReadConfigReg;
      { we mask off the read-only done flag...}
      if (configdata and $FFFFF7) = $000000 then Init4A23 := true;
      { now that we know things are working, set the mode and upper mux bits }
      { this is done by setting the desired output bits in the channel register}
      { and then transfering them to the aux register by setting the A-D CS}
      { high (inactive) and then low again}
      SetACSHigh;
      SetACSLow;
    end
    else
    writeln('Config Write-readback of 0xF00000 failed with ',configdata);
  end
  else  writeln('Card not found');
  { for now these are defaults }
  SetBipolarMode;
  SetPortFlagActive;
end;

procedure PowerDown4A23;
begin
  {Turn off the power}
  WriteSetupShadow(0);
  WriteDataShadow(0);
end;

function FastEOCQ : boolean;

begin
  if ADataHighQ then FastEOCQ := false else FastEOCQ := true;
end;

function EOCQ : boolean;
begin
  if (ReadConfigReg and ADDoneFlag) = 0 then EOCQ := false else EOCQ := true;
end;

function PolledReadAD : longint;
var
adata : longint;
begin
  ADSendCommand(SingleConvertCom);
  while not FastEOCQ do;
  adata := PFReadDataReg;
  adata := adata shr 4;
  if bipolar then
  begin
    if (adata > $80000) then adata := -($100000-adata);
  end;
  PolledReadAD := adata;
end;

function ISRReadAD : longint;
var
adata : longint;
begin
  adata := PFReadDataReg;
  adata := adata shr 4;
  if bipolar then
  begin
    if (adata > $80000) then adata := -($100000-adata);
  end;
  ISRReadAD := adata;
end;

function PolledReadADWithStatus(var rdata : longint) : boolean;
var
adata : longint;
goodconv : boolean;
begin
  goodconv := true;
  ADSendCommand(SingleConvertCom);
  while not FastEOCQ do;
  adata := PFReadDataReg;
  if (adata and $0000000F) <> 0 then  goodconv := false;
  adata := adata shr 4;
  if bipolar then
  begin
    if (adata > $80000) then adata := -($100000-adata);
  end;
  rdata := adata;
  PolledReadADWithStatus := goodconv;
end;

function ISRReadADWithStatus(var rdata : longint) : boolean;
var
adata : longint;
goodconv : boolean;
begin
  goodconv := true;
  adata := PFReadDataReg;
  if (adata and $0000000F) <> 0 then  goodconv := false;
  adata := adata shr 4;
  if bipolar then
  begin
    if (adata > $80000) then adata := -($100000-adata);
  end;
  rdata := adata;
  ISRReadADWithStatus := goodconv;
end;


function ReadAve(chan : byte; range: longint; times : word) : longint;
var
sumread : longint;
avetimes : word;
dummy : longint;
begin
  SetChannel(chan);
  SetRange(range);
  sumread := 0;
  for avetimes := 1 to times do
  begin
    sumread := sumread + PolledReadAD;
  end;
  ReadAve := sumread div times;
end;

function ReadCalData(calrecptr : CalRecPtrType) : boolean;
{ a sleazy way to access the calibrate record!  }
{ might be better to copy EEPROM data explicitly into }
{ the various fields instead of depending on    }
{ the specific (no padding) sequence in the record }
var
ereg : byte;
edata : word;
csum : word;
begin
  SetACSHigh;  {Turn off A-D CS so as not to confuse A-D}
  ReadCalData := true;
  csum := 0;
  for ereg := 0 to EEPROMSize-1 do
  begin
    if ReadSE(ereg,edata) then
    begin
      { Wierd type cast to be able to fill calibrate record with raw data }
      IndexedCalibRecordType(calrecptr^).words[ereg] := edata;
      csum := csum + edata;
    end
    else writeln ('EEPROM read failed!');
  end;
  if (calrecptr^.CRecType <> TheCRecType) then
  begin
    writeln('ReadCalData failed due to wrong record type');
    ReadCalData := false;
  end;
  if (csum <>0) then
  begin
    writeln('ReadCalData failed due to bad checksum:',csum);
    ReadCalData := false;
  end;
  SetACSLow;  {Turn A-D CS back on}
end;

function WriteCalData(calibrecptr : CalRecPtrType) : boolean;
{ a kludgy way to access the calibrate record!  }
{ might be better to copy EEPROM data explicitly from }
{ the various fields instead of depending on    }
{ the specific (no padding) sequence in the record }
{ an easy better way is to use a type cast on the CalibrateRecord}
var
ereg : byte;
edata : word;
csum : word;
begin
  WriteCalData := false;
  csum := 0;
  with calibrecptr^ do
  begin
    CheckSum := 0;       { poke 0 csum into the record }
    { first calculate checksum  }
    for ereg := 0 to EEPROMSize-1 do
    begin
      edata := IndexedCalibRecordType(calibrecptr^).words[ereg];
      csum := csum + edata;
    end;
    CheckSum := -csum;   { poke the complement of csum into the record }
    for ereg := 0 to EEPROMSize-1 do   { now write all the words of the EEPROM }
    begin
      edata := IndexedCalibRecordType(calibrecptr^).words[ereg];
      if not WriteSE(ereg,edata) then exit;
    end;
    WriteCalData := true;
  end;
end;

procedure PrintCalData(CalRecordPtr : CalRecPtrType);
var temp : real;
begin

  writeln('Card Revision level = ',CalRecordPtr^.CardRev);
  writeln('Calibrate Length = ',CalRecordPtr^.CRecLen,' Record Type   = ',CalRecordPtr^.CRecType);
  writeln('25 mV FS Gain = ',CalRecordPtr^.Gain25MV);
  writeln('50 mV FS Gain = ',CalRecordPtr^.Gain50MV);
  writeln('100 mV FS Gain = ',CalRecordPtr^.Gain100MV);
  writeln('1 V FS Gain = ',CalRecordPtr^.Gain1V);
  writeln('2.5 V FS Gain = ',CalRecordPtr^.Gain2r5V);
  writeln('5 V FS Gain = ',CalRecordPtr^.Gain5V);
  writeln('25 mV FS Offset = ',CalRecordPtr^.Offset25MV);
  writeln('50 mV FS Offset = ',CalRecordPtr^.Offset50MV);
  writeln('100 mV FS Offset = ',CalRecordPtr^.Offset100MV);
  writeln('1 V FS Offset = ',CalRecordPtr^.Offset1V);
  writeln('2.5 V FS Offset = ',CalRecordPtr^.Offset2r5V);
  writeln('5 V FS Offset = ',CalRecordPtr^.Offset5V);
  writeln('TempGain = ',CalRecordPtr^.TempGain);
  writeln('TempOffset = ',CalRecordPtr^.TempOff);
  temp := CalRecordPtr^.CalibrateTemp;
  temp := temp/TempScale;
  writeln('Calibrate temperature = ',temp:6:2,' C');
  writeln('Date calibrated = ',CalRecordPtr^.CalDateM,'/',CalRecordPtr^.CalDateD,'/',CalRecordPtr^.CalDateY);
  writeln('Number of times calibrated = ',CalRecordPtr^.CalTimes,' Last calibrated by ',CalRecordPtr^.CalBy);
end;



function  RunningAve(var accum : longint; new : longint; times: word) : longint;
var
{ accum is where the running average is stored, it is larger than the }
{ input data by the factor 'times' }
{ new is the new data to be averaged }
{ times is the inverse weight of the new data }
{ large values of 'times' will result in slow response times }
{ effectively a low pass filter }

tacc : longint;
begin
  tacc := accum + new;
  tacc := tacc * times;
  tacc := tacc div (times+1);
  accum := tacc;
  RunningAve := (tacc+(times div 2)) div times;
end;

function  ResettableRunningAve(var accum : longint; new : longint; times: word;step : word) : longint;
{ Because this running average is used chiefly as a noise filter, we don't }
{ average values that are > step counts different from our old value }
{ instead, we just reset the averager to the new value }
{ Note: To reduce calculations the value of the change is calculated using }
{ the accumulator, which is bigger that the reading by the factor 'times'}
{ this means that 'step' needs to be scaled by 'times' }
var
tacc : longint;
begin
  if abs((new * times) - accum) < step then
  begin
    tacc := accum + new;       { if new data is close to old then average }
    tacc := tacc * times;
    tacc := tacc div (times+1);
  end
  else
  begin
    tacc := new * times;       { otherwise reset averager to new value }
  end;
  accum := tacc;
  ResettableRunningAve := (tacc+(times div 2)) div times;
end;

function FSFromRange(range : longint) : real;
begin
  case range of
    ADRange100MV : FSFromRange := FullScaleVolts100mV;
    ADRange50MV  : FSFromRange := FullScaleVolts50mV;
    ADRange25MV  : FSFromRange := FullScaleVolts25mV;
    ADRange1V    : FSFromRange := FullScaleVolts1V;
    ADRange5V    : FSFromRange := FullScaleVolts5V;
    ADRange2r5V  : FSFromRange := FullScaleVolts2r5V;
    else FSFromRange := 0;
  end; {case}
end;

function AsVolts(data,range :longint) : real;
begin
    AsVolts := data*(FSFromRange(range)/FullScaleReal);
end;

function AsTemp(rawdata : longint;calptr : CalRecPtrType) : real;
var
temp : real;
begin                                  { This routine returns temp in degrees C }
  temp := AsVolts(RawData,ADRange1V) * calptr^.TempGain;
  temp := temp + calptr^.TempOff;      { not used yet - TempOff = 0 };
  temp := temp / TempScale;
  temp := temp - 273;                  { Kelvin to Celcius }
  AsTemp := temp;
end;
