Program AtoDBG; { setup A-D background conversions run by RTC }
{$R-}    {Range checking be off }
{$I-}    {No I/O checking }
{$S-}    {No stack checking}


uses DOS;
{$I general}
{$I 8259.pas}
{$I 4A23low}
const

TheInt   = 5;


var
DummyChar : word;
FooData : longint;
AtoDData : array [0..LastChannel] of longint;
AtoDDataBuffer : array [0..LastChannel] of longint;
AtoDIndex : byte;
Tindex : byte;
ReadyToRead : boolean;
OldInt : pointer;
CalibRec : CalibRecordType;

procedure UnHook;
begin
  MaskInterrupt(TheInt);
  DisableInterrupts;
  SetHardIntVec(TheInt,OldInt);
  EnableInterrupts;
end;

procedure Bumout(s:string);
begin
  writeln;
  writeln(s);
  writeln;
  halt(2);
end;

procedure ReadADDataISR;
interrupt;
begin
  { This masking is very important since the interrupt will  }
  { get repetitively set and cleared when reading the data   }
  { since it is basically just the A-D serial data out bit!  }
  { if we did not mask at the A-D here, we would immediately }
  { get another interrupt as soon as we re-enabled interrupts}
  MaskADInt;
  { next we read the data }
  AToDData[AToDIndex] := ISRReadAD;
  { then we set the channel for the next conversion}
  AToDIndex := AtoDIndex +1;
  if AToDIndex > LastChannel then
  { when we're done reading all channels into the working array, }
  { copy the working array into the buffer (AToDData >> AToDDataBuffer) }
  { and set a flag so that the foreground program knows it can read the }
  { data without it changing... better (faster) way would be to swap buffers}
  begin
    AToDIndex := 0;
    for Tindex := 0 to LastChannel do AToDDataBuffer[Tindex] := AToDData[Tindex];
    ReadyToRead := true;
  end;
  { set the channel number for the next conversion }
  SetChannel(AtoDIndex);
  { start next Conversion }
  ADSendCommand(SingleConvertCom);
  { unmask AD int at 4A23 }
  UnMaskADInt;
  { issue EOI to 8259}
  IssueEOI;
end;

procedure StartINTs;
begin
  writeln;
  if ReadCalData(@CalibRec) then
  begin
    SetInputSource(Normal);
    SetConvRate(ADWordRate3r76);
    SetRangeAndCal(ADRange5V,@CalibRec);
    SetChannel(0);
    {do dummy read}
    FooData := PolledReadAD;
  end
  else Bumout('Unable to read 4A23 calibration data');
  DisableInterrupts;
  MaskADInt;
  MaskInterrupt(TheInt);
  GetHardIntVec(TheInt,OldInt);
  SetHardIntVec(TheInt,@ReadADDataISR);
  if not SetIntNum(TheInt) then
  begin
    Unhook;
    BumOut('Invalid 4A23 Interrupt number');
  end;
  {start the ball rolling by sending a start convert command }
  ADSendCommand(SingleConvertCom);
  UnmaskInterrupt(TheInt);
  IssueEOI;
  UnMaskADInt;
  EnableInterrupts;
end;


(* procedure ReadLoop;
var
chan : byte;
adata : word;
sx,sy : byte;
msfree : word;
begin
  StartInts;
  while not keypressed do
  begin
    msfree:= 0;
    while not ReadyToRead do
    begin
      msfree := msfree+1;
      Localdelay(1);
    end; { spin (and count free time) until we have a full buffer of data }
    gotoXY(1,1);
    write('Free mS = ',msfree,'  ');
    ReadyToRead := false; { clear our flag from the ISR }
    for chan := 0 to 11 do
    begin
      adata := AToDDataBuffer[chan];  { GetData from array }
      sy := (chan div 2) +5;
      if (chan and 1) = 0 then sx := 1 else sx := 13;
      GoToXY(sx,sy);
      case chan of
        8: write('V =  ',(adata*VRef*2)/scale:2:2);
        9: write('T =  ',(0.660-(adata*VRef/scale))/0.0022:2:2);
        10: write('G =  ',adata div 16:4);
        11: write('CK = ',adata div 16:4);
        else write('Ch',chan:2,'=',adata div 16:4);
      end;
    end;
  end;
  UnHook;
  DummyChar := ReadKey;
end;
*)

procedure NonCRTReadLoop;
var
chan : byte;
sx,sy : byte;
msfree : word;
begin
  StartInts;
  while not keypressed do
  begin
    msfree:= 0;
    while not ReadyToRead do write;
    writeln;
    ReadyToRead := false; { clear our flag from the ISR }
    for chan := 0 to LastChannel do
    begin
      writeln('chan ',chan,': ',AtoDDataBuffer[chan]*FullScaleVolts5V/FullScaleReal :1:5);
    end;
  end;
  UnHook;
  DummyChar := ReadKey;
end;

begin { main }
  BasePort := A23BaseAdd0;
  if Init4A23 then
  begin
    NonCRTReadLoop;
  end
  else Writeln ('Problem with 4A23');
end. { main };




